#!/usr/bin/env python
#
# Copyright 2004,2005 Free Software Foundation, Inc.
#
# Copyright (c) 2006 BBN Technologies Corp.  All rights reserved.
# Effort sponsored in part by the Defense Advanced Research Projects
# Agency (DARPA) and the Department of the Interior National Business
# Center under agreement number NBCHC050166.
# 
# This file is part of GNU Radio
# 
# GNU Radio is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# GNU Radio is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with GNU Radio; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
# 

from gnuradio import gr, gru
from gnuradio import usrp2
from gnuradio import eng_notation
from gnuradio.eng_option import eng_option
import gnuradio.gr.gr_threading as _threading
from optparse import OptionParser
from bbn_80211b_pkt import *
from bbn_80211b import *
import sys
import struct
import os


class app_flow_graph(gr.top_block):
  def __init__(self, phy_rx_callback):
    gr.top_block.__init__(self)
        
    parser = OptionParser(option_class=eng_option)
    parser.add_option("-R", "--rx-subdev-spec", type="subdev", default=None, help="select USRP Rx side A or B (default=first one with a daughterboard)")
    parser.add_option("-d", "--decim", type="int", default=16, help="set fgpa decimation rate to DECIM [default=%default]")
    parser.add_option("-f", "--freq", type="eng_float", default=2.4e9, help="set frequency to FREQ", metavar="FREQ")
    parser.add_option("-g", "--gain", type="eng_float", default=None, help="set gain in dB (default is midpoint)")
    parser.add_option("", "--width-16", action="store_true", default=True, help="Enable 16-bit samples across USB")
    parser.add_option("-S", "--spb", type="int", default=8, help="set samples/baud [default=%default]")
    parser.add_option("-b", "--barker", action="store_true", default=False, help="Use Barker Spreading [default=%default]")
    parser.add_option("-p", "--no-crc-check", action="store_true", default=False, help="Check payload crc [default=%default]")
    parser.add_option("-v", "--verbose", action="store_true", default=False, help="Verbose Output")

    (options, args) = parser.parse_args()
    if len(args) != 0:
      parser.print_help()
      sys.exit(1)


    alpha = 0.001
    thresh = 30   # in dB, will have to adjust
    self.probe = gr.probe_avg_mag_sqrd_c(thresh,alpha)


    #self.u = usrp_rx(0, options.decim, options.rx_subdev_spec, options.width_16, options.verbose, options.gain, options.freq)
    self.u = usrp2.source_32fc("eth0", "")
    self.u.set_decim(options.decim * 3 / 2)

    if not options.width_16:
      width = 8
      shift = 8
      format = self.u.make_format(width, shift)
      r = self.u.set_format(format)
      if options.verbose:
        print "Bits Per Encoded Sample = 8"
    else:
      if options.verbose:
        print "Bits Per Encoded Sample = 16"
      

    if options.verbose:
      print "adc frequency = ", self.u.adc_rate()
      print "decimation frequency = ", self.u.decim()
      print "input_rate = ", self.u.adc_rate() / self.u.decim()

    if options.gain is None:
      # if no gain was specified, use the mid-point in dB
      g = self.u.gain_range()
      options.gain = float(g[0]+g[1])/2

    if options.verbose:
      print "gain = ", options.gain

    if options.freq is None:
      # if no freq was specified, use the mid-point
      r = self.u.freq_range()
      options.freq = float(r[0]+r[1])/2

    self.u.set_gain(options.gain)
    r = self.u.set_center_freq(options.freq)

    if options.verbose:
      print "desired freq = ", options.freq
      print "baseband frequency", r.baseband_freq
      print "dxc frequency", r.dxc_freq


    if options.verbose:
      print "Samples per data bit = ", options.spb
        
    self.bit_receiver = bbn_80211b_demod_pkts(spb=options.spb,
                                                  alpha=0.5,
						  callback=phy_rx_callback,
                                                  use_barker=options.barker,
                                                  check_crc=True)

 
    self.connect(self.u, self.bit_receiver)
    self.connect(self.u, self.probe)
    #self._watcher = _queue_watcher_thread(self.rcvd_pktq, rx_callback) 

  def carrier_sensed(self):
        """
        Return True if we think carrier is present.
        """
        #return self.probe.level() > X
        return self.probe.unmuted()

  def set_carrier_threshold(self, threshold_in_db):
        """
        Set carrier threshold.

        @param threshold_in_db: set detection threshold
        @type threshold_in_db:  float (dB)
        """
        self.probe.set_threshold(threshold_in_db)
   


def main ():
  app = app_flow_graph()
  app.start()
  os.read(0,10)

if __name__ == '__main__':
    main ()
